/*
 * simpletest for C:
 * - get instruction set and machine information;
 * - get C compiler and language standard version.
 *
 * Copyright © 2018-2020 Andrew Savchenko
 *
 * This file is a part of simpletest.
 *
 * simpletest is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation
 *
 * simpletest is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with simpletest. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <sys/utsname.h>

#if defined __e2k__
/* __builtin_cpu_is can't accept variables, only string literals */
char const * getarch(void)
{
    if (__builtin_cpu_is("elbrus-v1"))
        return "elbrus-v1";
    else if (__builtin_cpu_is("elbrus-v2"))
        return "elbrus-v2";
    else if (__builtin_cpu_is("elbrus-v3"))
        return "elbrus-v3";
    else if (__builtin_cpu_is("elbrus-v4"))
        return "elbrus-v4";
    else if (__builtin_cpu_is("elbrus-v5"))
        return "elbrus-v5";
    else if (__builtin_cpu_is("elbrus-v6"))
        return "elbrus-v6";
    else
        return "unknown";
}

char const * getcpu(void)
{
    if (__builtin_cpu_is("elbrus"))
        return "elbrus-v1";
    else if (__builtin_cpu_is("elbrus-2c+"))
        return "elbrus-2c+";
    else if (__builtin_cpu_is("elbrus-4c"))
        return "elbrus-4c";
    else if (__builtin_cpu_is("elbrus-8c"))
        return "elbrus-8c";
    else if (__builtin_cpu_is("elbrus-1c+"))
        return "elbrus-1c+";
    else if (__builtin_cpu_is("elbrus-8c2"))
        return "elbrus-8c2";
    else if (__builtin_cpu_is("elbrus-12c"))
        return "elbrus-12c";
    else if (__builtin_cpu_is("elbrus-16c"))
        return "elbrus-16c";
    else if (__builtin_cpu_is("elbrus-2c3"))
        return "elbrus-2c3";
    else
        return "unknown";
}
#endif

#if defined __i386__ || __x86_64__
// use gcc intrinsics to read CPUID registers
#include <cpuid.h>

char const * getcpu(void)
{
    static int a[13];
    __cpuid (0x80000002, a[0], a[1], a[2], a[3]);
    __cpuid (0x80000003, a[4], a[5], a[6], a[7]);
    __cpuid (0x80000004, a[8], a[9], a[10], a[11]);
    a[12]=0;
    return (char*)a;
}
#endif

int main() {
#if defined __e2k__
    printf("Current CPU arch:\t\t%s\nCurrent CPU model:\t\t%s\n\n", getarch(), getcpu());
#if defined __iset__
    printf("Built using instruction set:\t%i\n", __iset__);
    printf("Built for mode:\t\t\t%s\n",
#ifdef __ptr32__
            "ptr32"
#elif __ptr64__
            "ptr64"
#elif __ptr128__
            "ptr128"
#else
            "unknown"
#endif
    );
#endif
#endif

#if defined __i386__ || __x86_64__
    printf("Current CPU model:\t\t%s\n", getcpu());
#endif
    int opt = 0;
#ifdef __OPTIMIZE__
    opt = __OPTIMIZE__;
#endif
    printf("Optimization state:\t\t%i\n\n", opt);

    struct utsname un;
    uname(&un);

    printf("sysname:\t%s\nnodename:\t%s\nrelease:\t%s\nversion:\t%s\nmachine:\t%s\n\n",
        un.sysname, un.nodename, un.release, un.version, un.machine);
#if defined __VERSION__
    printf("Compatible compiler version:\t%s\n", __VERSION__);
#endif
#if defined __STDC_VERSION__
    printf("Supported C standard version:\t%li\n", __STDC_VERSION__);
#endif

    return 0;
}
